# Author: Koushik Sen (ksen@berkeley.edu)
# Contributors:
# Koushik Sen (ksen@berkeley.edu)
# add your name here
"""Docker wrapper module for the KISS agent framework."""

from kiss.docker.docker_manager import DockerManager

__all__ = ["DockerManager"]
